@@ -5,46 +5,60 @@ package Module::Install::ExtraTests;
 use Module::Install::Base;
 
 BEGIN {
-  our $VERSION = '0.006';
+  our $VERSION = '0.007';
   our $ISCORE  = 1;
   our @ISA     = qw{Module::Install::Base};
 }
 
+our $use_extratests = 0;
+
 sub extra_tests {
   my ($self) = @_;
 
   return unless -d 'xt';
-  return unless my @content = grep { $_ =~ /^[.]/ } <xt/*>;
+  return unless my @content = grep { $_ !~ /^[.]/ } <xt/*>;
 
-  die "unknown files found in ./xt" if grep { -f } @content;
+  die "unknown files found in ./xt" if grep { !-d } @content;
 
-  my %known   = map {; $_ => 1 } qw(author smoke release);
+  my %known   = map {; "xt/$_" => 1 } qw(author smoke release);
   my @unknown = grep { not $known{$_} } @content;
   die "unknown directories found in ./xt: @unknown" if @unknown;
 
-  {
-    no warnings qw(closure once);
-    package # The newline tells PAUSE, "DO NOT INDEXING!"
-    MY;
-    sub test_via_harness {
-      my ($self, $perl, $tests) = @_;
-      my $a_str = -d 'xt/author'  ? 'xt/author'  : '';
-      my $r_str = -d 'xt/release' ? 'xt/release' : '';
-      my $s_str = -d 'xt/smoke'   ? 'xt/smoke'   : '';
-      my $is_author = $Module::Install::AUTHOR ? 1 : 0;
+  $use_extratests = 1;
 
-      return qq{\t$perl "-Iinc" "-MModule::Install::ExtraTests" }
-           . qq{"-e" "Module::Install::ExtraTests::__harness('Test::Harness', $is_author, '$a_str', '$r_str', '$s_str', \$(TEST_VERBOSE), '\$(INST_LIB)', '\$(INST_ARCHLIB)')" $tests\n};
-    }
+  return;
+}
 
-    sub dist_test {
-      my ($self, @args) = @_;
-      my $text = $self->SUPER::dist_test(@args);
-      my @lines = split /\n/, $text;
-      $_ =~ s/ (\S*MAKE\S* test )/ RELEASE_TESTING=1 $1 / for grep { m/ test / } @lines;
-      return join "\n", @lines;
-    }
+{
+  no warnings qw(once);
+  package # The newline tells PAUSE, "DO NOT INDEXING!"
+  MY;
+  sub test_via_harness {
+    my $self = shift;
 
+    return $self->SUPER::test_via_harness(@_)
+      unless $use_extratests;
+
+    my ($perl, $tests) = @_;
+    my $a_str = -d 'xt/author'  ? 'xt/author'  : '';
+    my $r_str = -d 'xt/release' ? 'xt/release' : '';
+    my $s_str = -d 'xt/smoke'   ? 'xt/smoke'   : '';
+    my $is_author = $Module::Install::AUTHOR ? 1 : 0;
+
+    return qq{\t$perl "-Iinc" "-MModule::Install::ExtraTests" }
+         . qq{"-e" "Module::Install::ExtraTests::__harness('Test::Harness', $is_author, '$a_str', '$r_str', '$s_str', \$(TEST_VERBOSE), '\$(INST_LIB)', '\$(INST_ARCHLIB)')" $tests\n};
+  }
+
+  sub dist_test {
+    my ($self, @args) = @_;
+
+    return $self->SUPER::dist_test(@args)
+      unless $use_extratests;
+
+    my $text = $self->SUPER::dist_test(@args);
+    my @lines = split /\n/, $text;
+    $_ =~ s/ (\S*MAKE\S* test )/ RELEASE_TESTING=1 $1 / for grep { m/ test / } @lines;
+    return join "\n", @lines;
   }
 }
 
